/*
*************************************************************
                                                                 
       Demand Prediction and Welfare Analysis Code
            of MDCEV with Triple Constraints    
                          v1.00

                        Written by 

                      Koich Kuriyama

            Division of Natural Resource Economics,
                Graduate School of Agriculture,
                      Kyoto University
                  Oiwake-cho, Kitashirakawa, 
                Sakyo-ku, Kyoto 606-8502, JAPAN
                 E-mail kkuri@kais.kyoto-u.ac.jp
      Web: http://kkuri.eco.coocan.jp/index-e.html

                         07/28/2016

*************************************************************
*/

new , 40000;
outwidth 256;
screen on;
cls;

@--------------------------@
@      SETTING START       @
@--------------------------@

@ Output File Setting @
output file = DEMAND_TC3.out reset;

@ Loading data @
load DATA[100,25] = data.txt; @ Data File @

@ Number of observations @
N_OBS = rows(DATA);

@ Number of inside goods (excluding the outside goods) @
N_INSIDE = 3;

@ CHOICE SET of inside goods (1: include, 0: not include) @
CSET_INSIDE = ones(N_OBS,N_INSIDE);

@ Nubmer of Psi variables for inside goods @
N_PSI = 3; 

@ Number of constraints (= Number of outside goods) @
N_CONSTRAINTS = 3;    @ 1: budget, 2: budget + time, 3: budget + time1 + time2 @


@--------------------------@
@ PREFERENCE SPECIFICATION @
@--------------------------@

@ Alpha: 4 x 1, if ALPHAKFIX = 0. (3 + N_INSIDE) x 1, if ALPHAKFIX = 1. @
@  1: estimate, 0: not estimate (alpha = 0), -1: estimated alphas are constrained to be equal to alphak @
ALPHA_EST = {
	1, @ alhpa1 (money) @
 	0, @ alpha2 (time1: weekend) @
	0, @ alpha3 (time2: long holiday) @
	0  @ alphak of inside goods @
}; 

@ gamma: 1 x 1, if GAMMAKFIX = 0. N_INSIDE x 1, if GAMMAKFIX = 1. @
@  1: estimate, 0: not estimate (gammak = 1) @
GAMMA_EST = {
	1  @ gammak of inside goods @
};     @ gamma1, gamma2, and gamma3 are always fixed at 0 @

@ Sigma: 4 x 1, if SIGKFIX = 0. (3 + N_INSIDE) x 1, if SIGKFIX = 1. @
@  1: estimate, 0: not estimate (sig = 0), -1: estimated sigmas are constrained to be equal to sigk @
SIG_EST = {
	1, @ sig1 @
	0, @ sig2 @
	0  @ sig3 @
};     @ sigk of inside goods is always estimated @

@ XI ESTIMATION (3 x 1) @
@  1: estimate, 0: not estimate (use 0) @
XI_EST = {
	0, @ xi1: money @
	1, @ xi2: time of the weekend @
	1  @ xi3: time of the long holiday @
}; 

@ OTHER SETTING OF PARAMETERS @
@ 1: good specific fixed effect, 0: equal to all indside goods @
ALPHAKFIX = 0; @ 1: alphak is fixed effect, 0: otherwise @
GAMMAKFIX = 0; @ 1: gammak is fixed effect, 0: otherwise @
SIGKFIX = 0;   @ 1: sigk is fixed effect, 0: otherwise @

@--------------------------@
@    ESTIMATED VALUES      @
@--------------------------@

B = {
  -4.528910e+000 ,
   4.222466e-001 ,
  -3.131753e-001 ,
   1.000000e-012 ,
   1.874035e+000 ,
  -1.480289e+000 ,
  -3.887972e-001 ,
   1.000000e-012 ,
   1.000000e-012 ,
   1.000000e-012 ,
   1.052661e+000 ,
   1.044201e+000 ,
   1.000000e-012 ,
   1.000000e-012 ,
  -2.507203e+000
};

VCOV = {
   1.200595e-001   -2.581419e-003    3.720092e-003                .    1.068355e-001    7.208293e-002   -1.440636e-002                .                .                .    1.306079e-003   -7.691763e-003                .                .   -5.526055e-003 ,
  -2.581419e-003    1.513845e-004   -1.582190e-004                .   -1.734433e-003   -1.037286e-003    2.566030e-004                .                .                .   -1.083058e-004    1.199619e-004                .                .    1.857479e-004 ,
   3.720092e-003   -1.582190e-004    3.571256e-004                .    3.220117e-003    1.884267e-003   -4.662673e-004                .                .                .    9.069733e-005   -2.130240e-004                .                .   -3.717885e-004 ,
               .                .                .                .                .                .                .                .                .                .                .                .                .                .                . ,
   1.068355e-001   -1.734433e-003    3.220117e-003                .    1.038498e-001    7.334095e-002   -1.338422e-002                .                .                .    2.965759e-003   -7.333400e-003                .                .   -6.628890e-003 ,
   7.208293e-002   -1.037286e-003    1.884267e-003                .    7.334095e-002    9.200654e-002   -9.328624e-003                .                .                .    4.933736e-003   -4.404828e-003                .                .   -4.038336e-003 ,
  -1.440636e-002    2.566030e-004   -4.662673e-004                .   -1.338422e-002   -9.328624e-003    1.819009e-003                .                .                .   -2.552093e-004    9.135273e-004                .                .    7.249439e-004 ,
               .                .                .                .                .                .                .                .                .                .                .                .                .                .                . ,
               .                .                .                .                .                .                .                .                .                .                .                .                .                .                . ,
               .                .                .                .                .                .                .                .                .                .                .                .                .                .                . ,
   1.306079e-003   -1.083058e-004    9.069733e-005                .    2.965759e-003    4.933736e-003   -2.552093e-004                .                .                .    2.400829e-003   -2.441040e-004                .                .   -6.031744e-004 ,
  -7.691763e-003    1.199619e-004   -2.130240e-004                .   -7.333400e-003   -4.404828e-003    9.135273e-004                .                .                .   -2.441040e-004    6.836139e-004                .                .    3.792057e-004 ,
               .                .                .                .                .                .                .                .                .                .                .                .                .                .                . ,
               .                .                .                .                .                .                .                .                .                .                .                .                .                .                . ,
  -5.526055e-003    1.857479e-004   -3.717885e-004                .   -6.628890e-003   -4.038336e-003    7.249439e-004                .                .                .   -6.031744e-004    3.792057e-004                .                .    9.595756e-003 
};


@--------------------------@
@       OTHER SETTING      @
@--------------------------@

@ Number of total iteration of simulation @
N_SIM = 2500;

@ Number of Burn-in for simulation @
N_BURNIN = 500;

@ Keep every nth simulated values after burn-in @
SKIP1 = 10;

@ Number of iteration of Krinsky-Robb simulation @
N_KR = 1; @ N_KR=1: point estimates, N_KR>1: KR simulation @

@ Random seed @
NSEED0_ORG = 683496;
NSEED1_ORG = 750516 ;
NSEED2_ORG = 92721 ;
NSEED3_ORG = 746592 ;
NSEED4_ORG = 228803 ;

@ Loading data @
/*
Data format for N_INSIDE = 3
  1:    person id 
  2- 4: X_Inside_S: consumption of inside goods in weekends
  5- 7: X_Inside_L: consumption of inside goods in long holidays
  8- 10: psi1 (const.)
 11- 13: psi2
 14- 16: psi3
 17:     income
 18- 20: price
 21:     Max_TS
 22:     Max_TL
 23- 25: time
*/
IDX = 2;
XINSIDE_S = DATA[.,IDX:(IDX+N_INSIDE-1)]; @ X_Inside_S @
IDX = IDX + N_INSIDE;
XINSIDE_L = DATA[.,IDX:(IDX+N_INSIDE-1)]; @ X_Inside_L @
IDX = IDX + N_INSIDE;
PSIDATA = DATA[.,IDX:(IDX+N_INSIDE.*N_PSI-1)]; @ Psi @
IDX = IDX + N_INSIDE.*N_PSI;
INCOME = DATA[.,IDX];  @ Income @
IDX = IDX + 1;
PRICE = DATA[.,IDX:(IDX+N_INSIDE-1)]; @ Price @
IDX = IDX + N_INSIDE;
MAX_TS = DATA[.,IDX]; @ Max_Time of x2 (weekend) @
IDX = IDX + 1;
MAX_TL = DATA[.,IDX]; @ Max_Time of x3 (long holiday) @
IDX = IDX + 1;
TTIME = DATA[.,IDX:(IDX+N_INSIDE-1)]; @ two-way trip time @

PSIDATA_S = PSIDATA; @ weekend @
PSIDATA_L = PSIDATA; @ long holiday, not used for single or double constraints @
PRICE_S = PRICE; @ weekend @
PRICE_L = PRICE; @ long holiday, not used for single or double constraints @
TTIME_S = TTIME; @ weekend @
TTIME_L = TTIME; @ long holiday, not used for single or double constraints @
CSET_INSIDE_S = CSET_INSIDE; @ weekend @
CSET_INSIDE_L = CSET_INSIDE; @ long holiday, not used for single or double constraints @

@--------------------------@
@         SCENARIOS        @
@--------------------------@
@ Scenario #1: Add five days in a weekend  @

PSIDATA_S1 = PSIDATA_S;
PSIDATA_L1 = PSIDATA_L;
PRICE_S1 = PRICE_S;
PRICE_L1 = PRICE_L;
MAX_TS1 = MAX_TS + 5 * 24;
MAX_TL1 = MAX_TL;
TTIME_S1 = TTIME_S;
TTIME_L1 = TTIME_L;
CSET_INSIDE_S1  = CSET_INSIDE_S;
CSET_INSIDE_L1  = CSET_INSIDE_L;

@ Scenario #2: Add five days in a long vacation @
PSIDATA_S2 = PSIDATA_S;
PSIDATA_L2 = PSIDATA_L;
PRICE_S2 = PRICE_S;
PRICE_L2 = PRICE_L;
MAX_TS2 = MAX_TS;
MAX_TL2 = MAX_TL + 5 * 24;
TTIME_S2 = TTIME_S;
TTIME_L2 = TTIME_L;
CSET_INSIDE_S2  = CSET_INSIDE_S;
CSET_INSIDE_L2  = CSET_INSIDE_L;

@ Scenario #3: Closure of site #1 in a long holiday@
PSIDATA_S3 = PSIDATA_S;
PSIDATA_L3 = PSIDATA_L;
PRICE_S3 = PRICE_S;
PRICE_L3 = PRICE_L;
PRICE_L3[.,1] = 100000000*ones(N_OBS,1);
MAX_TS3 = MAX_TS;
MAX_TL3 = MAX_TL;
TTIME_S3 = TTIME_S;
TTIME_L3 = TTIME_L;
CSET_INSIDE_S3  = CSET_INSIDE_S;
CSET_INSIDE_L3  = CSET_INSIDE_L;

@--------------------------@
@        SETTING END       @
@--------------------------@
HICKSP = ones(N_OBS,1); 
N_INSIDE_ORG = N_INSIDE;
PSIDATA_ORG = PSIDATA;
if N_CONSTRAINTS .== 3;
     tmp0 = zeros(N_OBS,1);
	 tmp1 = zeros(N_OBS,1);
	 tmp2 = zeros(N_OBS,1);
	 tmp3 = zeros(N_OBS,1);
     k = 1;
     do while k .le N_PSI;
         idx = (k-1).*N_INSIDE+1;
         idx2 = idx + N_INSIDE - 1;
         tmp0 = tmp0~PSIDATA_S[.,idx:idx2]~PSIDATA_L[.,idx:idx2];
         tmp1 = tmp1~PSIDATA_S1[.,idx:idx2]~PSIDATA_L1[.,idx:idx2];
         tmp2 = tmp2~PSIDATA_S2[.,idx:idx2]~PSIDATA_L2[.,idx:idx2];
         tmp3 = tmp3~PSIDATA_S3[.,idx:idx2]~PSIDATA_L3[.,idx:idx2];
         k = k + 1;
    endo;
    XINSIDE = XINSIDE_S~XINSIDE_L;
    PSIDATA = tmp0[.,2:cols(tmp0)];
    PSIDATA1 = tmp1[.,2:cols(tmp1)];
	PSIDATA2 = tmp2[.,2:cols(tmp2)];
    PSIDATA3 = tmp3[.,2:cols(tmp3)];
    PRICE = PRICE~PRICE;
    PRICE1 = PRICE_S1~PRICE_L;
    PRICE2 = PRICE_S2~PRICE_L2;
    PRICE3 = PRICE_S3~PRICE_L3;
    TTIME = TTIME~TTIME;
    TTIME1 = TTIME_S1~TTIME_L1;
    TTIME2 = TTIME_S2~TTIME_L2;
    TTIME3 = TTIME_S3~TTIME_L3;
    CSET_INSIDE = CSET_INSIDE~CSET_INSIDE;
    CSET_INSIDE1 = CSET_INSIDE_S1~CSET_INSIDE_L1;
    CSET_INSIDE2 = CSET_INSIDE_S2~CSET_INSIDE_L2;
    CSET_INSIDE3 = CSET_INSIDE_S3~CSET_INSIDE_L3;
    SK = ones(N_OBS,N_INSIDE)~zeros(N_OBS,N_INSIDE);
    SK2 = 1 - SK;
    N_INSIDE = N_INSIDE*2;
else;
    XINSIDE = XINSIDE_S + XINSIDE_L;
    PSIDATA1 = PSIDATA_S1;
    PSIDATA2 = PSIDATA_S2;
    PSIDATA3 = PSIDATA_S3;
    PRICE1 = PRICE_S1;
    PRICE2 = PRICE_S2;
    PRICE3 = PRICE_S3;
    TTIME1 = TTIME_S1;
    TTIME2 = TTIME_S2;
    TTIME3 = TTIME_S3;
    CSET_INSIDE1 = CSET_INSIDE_S1;
    CSET_INSIDE2 = CSET_INSIDE_S2;
    CSET_INSIDE3 = CSET_INSIDE_S3;	
    SK = ones(N_OBS,N_INSIDE);
    SK2 = zeros(N_OBS,N_INSIDE);
endif;    
XOUT1 = INCOME - sumc((PRICE.*XINSIDE)'); @ Hicks composite good of money @


struct util_data {
    matrix demand;
    matrix price;
    matrix inc;
    matrix alpha1;
    matrix alpha2;
    matrix alpha3;
    matrix alphak;
    matrix gammak;
    matrix psik;
    matrix xi1;
    matrix xis;
    matrix xil;
    matrix maxts;
    matrix maxtl;
    matrix tt;
    matrix keep;
    matrix eps;
};

struct demand_data {
    matrix psi;
    matrix price;
    matrix inc;
    matrix util0;
    matrix alpha1;
    matrix alpha2;
    matrix alpha3;
    matrix alphak;
    matrix gamma;
    matrix keep;
    matrix hprice;
    matrix eps;
    matrix maxts;
    matrix maxtl;
    matrix xi1;
    matrix xis;
    matrix xil;
    matrix tt;
    matrix lambda;
};

@ Variables for qsolve @
    dd_psin = 0;
    dd_pricen = 0;
    dd_alpha1n = 0;
    dd_alpha2n = 0;
    dd_alpha3n = 0;
    dd_alphakn = 0;
    dd_xi1n = 0;
    dd_xisn = 0;
    dd_xiln = 0;
    dd_ttn = 0;
    dd_skn = 0;
    dd_gamman = 0;
    dd_srepn = 0;
    dd_incn = 0;
    dd_maxtsn = 0;
    dd_maxtln = 0;
    dd_keepn = 0;
    dd_util0n = 0;
    dd_errb1n = 0;
    dd_eps1n = 0;
    dd_eps2n = 0;
    dd_eps3n = 0;

sig_RAND = (sig_EST .== 1);
@ BACTIVE = 1 if estimated, 0 if fixed @
BACTIVE = ones(rows(b),1);
IDX1 = N_PSI + 1;
IDX2 = IDX1 + rows(XI_EST) - 1;
if N_CONSTRAINTS == 1;
    XI_DUMMY = {1,0,0};
    SC_MODEL = 1;
    DC_MODEL = 0;
    TC_MODEL = 0;
elseif N_CONSTRAINTS == 2;
    XI_DUMMY = {1,1,0};
    SC_MODEL = 0;
    DC_MODEL = 1;
    TC_MODEL = 0;
elseif N_CONSTRAINTS == 3;
    XI_DUMMY = {1,1,1};
    SC_MODEL = 0;
    DC_MODEL = 0;
    TC_MODEL = 1;
endif;    
BACTIVE[IDX1:IDX2] = (XI_EST.==1).*XI_DUMMY;  
IDX1 = IDX2 + 1;
IDX2 = IDX1 + rows(ALPHA_EST) - 1;
BACTIVE[IDX1:IDX2] = (ALPHA_EST.==1);  
BACTIVE[IDX1:(IDX1+2)] = BACTIVE[IDX1:(IDX1+2)].*XI_DUMMY;  
IDX1 = IDX2 + 1;
IDX2 = IDX1 + rows(GAMMA_EST) - 1;
BACTIVE[IDX1:IDX2] = (GAMMA_EST.==1);
IDX1 = IDX2 + 1;
IDX2 = IDX1 + rows(SIG_EST) - 1;
BACTIVE[IDX1:(IDX1+2)] = (SIG_EST.==1);
BACTIVE[IDX1:(IDX1+2)] = BACTIVE[IDX1:(IDX1+2)].*XI_DUMMY;  


if N_KR == 1;
   NSEED1 = NSEED1_ORG;
   NSEED2 = NSEED2_ORG;
   NSEED3 = NSEED3_ORG;
   startt = hsec;
   { WTP, DEMAND, DEMAND2, LAMBDAB } = wtp1(B);

   stopt  = hsec;
   print "******************************************************************";
   print "              Point Estimates of Demand, WTP, and VLT            ";
   print "******************************************************************";
   print;
   print "Time (in minutes):                 " ((-startt+stopt)/6000);
   print "Total Genreated Draws:              " N_SIM;
   print "Discarded as Burn-in:              " N_BURNIN;
   print "Using Every nth Draw,          n =:" SKIP1;
   print "Total Draws Used for Solving:      " ((N_SIM-N_BURNIN)/SKIP1);
   print;				
   print;				
   print "Mean for Demand";
   print "       BASELINE         SCENARIO #1      SCENARIO #2      SCENARIO #3";   
   print meanc(DEMAND)';
   print;
   print;				
   print "       GOODS            BASELINE         SCENARIO #1      SCENARIO #2      SCENARIO #3";   
   TEMP = (seqa(1,1,N_INSIDE))~(meanc(DEMAND2[.,1:N_INSIDE]))~(meanc(DEMAND2[.,(N_INSIDE+1):(N_INSIDE*2)]))
   ~(meanc(DEMAND2[.,(N_INSIDE*2+1):(N_INSIDE*3)]))~(meanc(DEMAND2[.,(N_INSIDE*3+1):(N_INSIDE*4)]));
   print TEMP;
   print;
   print;				
   print "Mean for WTP";
   print "       SCENARIO #1      SCENARIO #2      SCENARIO #3";   
   print meanc(WTP[.,1:3])';
   print;
   print;				
   print "Mean for Lambda1      Lambda2         Lambda3             VLTS             VLTL";   
   print meanc(LAMBDAB)';
else;
   KR1 = zeros(N_KR,7);
   KR2 = zeros(N_KR,N_INSIDE*4);
   KR3 = zeros(N_KR,5);
   KR4 = zeros(N_KR,16);
   startt = hsec;
   loop_KR = 1;
   do while loop_KR <= rows(KR1);
      print "loop: " loop_KR;
      NSEED1 = NSEED1_ORG;
      NSEED2 = NSEED2_ORG;
      NSEED3 = NSEED3_ORG;      
      dum = (BACTIVE .== 0);
      BB = delif(b,dum);
      VCOVB = delif(VCOV,dum);
      VCOVB = delif(VCOVB',dum);
      b11 = BB + (chol(VCOVB)')*rndns(rows(BB),1,NSEED4_ORG);
      b1 = b;
      k = 1;
      idx = 1;
      do while k .le rows(b);
          if dum[k] == 0;
            b1[k] = b11[idx];
            idx = idx + 1;
          endif;
          k = k + 1;
      endo;     
      { WTP, DEMAND, DEMAND2, LAMBDAB } = wtp1(b1);
      KR1[loop_KR,1:3] = meanc(WTP)';
      KR1[loop_KR,4:7] = meanc(DEMAND)';  
      KR2[loop_KR,.] =  meanc(DEMAND2)';   
      KR3[loop_KR,1:3] = meanc(LAMBDAB[.,1:3])';  
      KR3[loop_KR,4] = meanc(LAMBDAB[.,4]);  
      KR3[loop_KR,5] = meanc(LAMBDAB[.,5]); 
      if ismiss(meanc(WTP));
          print "miss";
      else;
        loop_KR = loop_KR + 1;
      endif;	  
   endo;
   stopt = hsec;
   print "******************************************************************";
   print "              Krinsky-Robb Monte Carlo Simulation                 ";
   print "******************************************************************";
   print "Time (in minutes):                 " ((-startt+stopt)/6000);
   print "Number of Krinsky-Robb Iterations: " rows(KR1);
   print "Total Generated Draws:              " N_SIM;
   print "Discarded as Burn-in:              " N_BURNIN;
   print "Using Every nth Draw,          n =:" SKIP1;
   print "Total Draws Used for Solving:      " ((N_SIM-N_BURNIN)/SKIP1);
   print;
   print;				

   KR_ALL = KR1~KR3~KR2~KR4; @ Not Soring Result of KR1 @
   wtpl1 = zeros(cols(KR1),1);
   wtpu1 = zeros(cols(KR1),1);
   k = 1;
   do while k <= cols(KR1);
      {wtpl1[k],wtpu1[k]} = confint(KR1[.,k]);
      k = k + 1;
   endo;

   wtpl2 = zeros(cols(KR2),1);
   wtpu2 = zeros(cols(KR2),1);
   k = 1;
   do while k <= cols(KR2);
      {wtpl2[k],wtpu2[k]} = confint(KR2[.,k]);
      k = k + 1;
   endo;

   wtpl3 = zeros(cols(KR3),1);
   wtpu3 = zeros(cols(KR3),1);
   k = 1;
   do while k <= cols(KR3);
      {wtpl3[k],wtpu3[k]} = confint(KR3[.,k]);
      k = k + 1;
   endo;   
   
   print "Demand (Scenario #0 means the baseline)";
   print "       SCENARIO           MEAN            SD               95%lower         95%upper";   
   print seqa(0,1,4)~meanc(KR1[.,4:7])~stdc(KR1[.,4:7])~wtpl1[4:7]~wtpu1[4:7];
   print;				
   print;				
   print "Demand for Each Good (Scenario #0 means the baseline)";
   print "       SCENARIO           GOOD              MEAN            SD             95%lower         95%upper";   
   print (reshape(seqa(0,1,4).*ones(4,N_INSIDE),4.*N_INSIDE,1)
        ~(seqa(1,1,N_INSIDE)|seqa(1,1,N_INSIDE)|seqa(1,1,N_INSIDE)|seqa(1,1,N_INSIDE))
        ~meanc(KR2)~stdc(KR2)~wtpl2~wtpu2);
   print;				
   print;				
   print "WTP";
   print "       SCENARIO           MEAN            SD               95%lower         95%upper";   
   print seqa(1,1,3)~meanc(KR1[.,1:3])~stdc(KR1[.,1:3])~wtpl1[1:3]~wtpu1[1:3];
   print;				
   print;
   print "Lambda and Time Value";
   print "#1:lambda1, #2:lambda2, #3:lambda3, #4:VLTS, #5:VLSL";
   print "           #             MEAN             SD             95%lower           95%upper";   
   print seqa(1,1,5)~meanc(KR3)~stdc(KR3)~wtpl3~wtpu3;
endif;

@ Prediction of the demands, compensating variations and VLTs @

proc (4) = wtp1(bn);
    local alpha1, alpha2, alpha3, alphak, gammak;
    local sig1, sig2, sig3, sigk, sig123;
    local ppsi, psikn, psi1n, psi2n, psi3n, psik0, psi1, psi2, psi3;
    local wtpall, wtp1i, wtp2i, wtp3i;
    local demandall, demandall2, demandbi, demand1i, demand2i, demand3i;
    local utilb, gumbelk0, gumbelk1;
    local rho, stat, thresh, pdf0, pdf1;
    local errc0, erre0, erre1, errk;
    local likek0, likek1, like1;
    local hlambda1i, hlambda2i, hlambda3i;
    local lambdabi, lambda1i,lambda2i,lambda3i;
    local eps1,eps2,eps3, eps;
    local xi1, xis, xil, bsk;
    local x2i0, x3i0;
    local det0, det1, det2;
    local tsk, s, k, k2, x2i, x3i, like;
    local lambdab1, lambdab2, lambdab3, lambdaball, VOLB1, VOLB2;
    local retcode, retflag;
    
    @ Loading parameters @
    k = 1;
    @ PSI @
    if N_PSI > 0;
        ppsi = bn[k:(k+N_PSI-1)];
        k = k + N_PSI;
    else;
        ppsi  = 0;
    endif;

    @ XI @
    xi1 = (XI_EST[1] /= 1).* 0 + (XI_EST[1] ==1) .* (bn[k]);  
    k = k + 1;
    xis = (XI_EST[2] /= 1).* 0 + (XI_EST[2] ==1) .* (bn[k]);  
    k = k + 1;
    xil = (XI_EST[3] /= 1).* 0 + (XI_EST[3] ==1) .* (bn[k]);  
    k = k + 1;  

    @ ALPHA @
    alpha1 = 1-exp(bn[k]);  
    k = k + 1;
    alpha2 = 1-exp(bn[k]);  
    k = k + 1;
    alpha3 = 1-exp(bn[k]);  
    k = k + 1;
    if ALPHAKFIX == 1;
        alphak = (1-exp(bn[k:(k+N_INSIDE_ORG-1)]))'; 
		alphak = alphak~alphak;
        k = k + N_INSIDE_ORG;
    else;
        alphak = 1-exp(bn[k]);
        k = k + 1;
    endif;
  
    alpha1 = (alpha_EST[1] == 0).* 0 + (alpha_EST[1] == -1).* alphak[1] + (alpha_EST[1] ==1) .* alpha1;
    alpha2 = (alpha_EST[2] == 0).* 0 + (alpha_EST[2] == -1).* alphak[1] + (alpha_EST[2] ==1) .* alpha2;
    alpha3 = (alpha_EST[3] == 0).* 0 + (alpha_EST[3] == -1).* alphak[1] + (alpha_EST[3] ==1) .* alpha3;
    alphak = (alpha_EST[4] == 0).* 0 + (alpha_EST[4] ==1) .* alphak;

    @ GAMMA @
    if GAMMAKFIX == 1;
		gammak = exp(bn[k:(k+N_INSIDE_ORG-1)])';
		gammak = (gammak~gammak).*ones(N_OBS,N_INSIDE);
        gammak = ((gamma_EST|gamma_EST)' ./= 1).* ones(N_OBS,N_INSIDE)
		       + ((gamma_EST|gamma_EST)' .==1) .* gammak;
        k = k + N_INSIDE_ORG;
    else;
        gammak = exp(bn[k]).*ones(N_OBS,N_INSIDE);
        k = k + 1;
        gammak = (gamma_EST[1] /= 1).* ones(N_OBS,N_INSIDE) + (gamma_EST[1] ==1) .* gammak;
    endif;
    
    @ SIGMA @
    sig1 = exp(bn[k]);
    sig2 = exp(bn[k+1]);
    sig3 = exp(bn[k+2]);
    k = k + 3;

    if SIGKFIX /= 1;
        sigk = exp(bn[k].*ones(1,N_INSIDE));
        k = k + 1;
    else;
        sigk = exp(bn[k:(k+N_INSIDE-1)]');
        k = k + N_INSIDE;
    endif;  
  
    sig1 = (sig_EST[1] == -1).* sigk[.,1] + (sig_EST[1] == 0).* 1 + (sig_EST[1] ==1) .* sig1;
    sig2 = (sig_EST[2] == -1).* sigk[.,1] + (sig_EST[2] == 0).* 1 + (sig_EST[2] ==1) .* sig2;
    sig3 = (sig_EST[3] == -1).* sigk[.,1] + (sig_EST[3] == 0).* 1 + (sig_EST[3] ==1) .* sig3;
    sig123 = sig1 | sig2 | sig3;
        
    x2i = Max_TS - sumc((TTIME.*sk.*XINSIDE)'); @ x2i: outside demand of a weekend  @
    x3i = Max_TL - sumc((TTIME.*(1-sk).*XINSIDE)'); @ x3i: outside demand of a long holiday  @
    
@ The Metropolis-Hasgings Algoritm @
  
    rho = .1;
    errc0 = zeros(N_OBS,N_PSI); 
    erre0 = zeros(N_OBS,rows(sig123));
    pdf0 = ones(N_OBS,1);     
    k = 1;
    do while k <= rows(sig123);
        pdf0 = pdf0.*(sig_RAND[k] == 1)
            .*1./sig123[k].*exp(-erre0[.,k]./sig123[k])
            .*(exp(-exp(-erre0[.,k]./sig123[k]) ))
            + pdf0.*(sig_RAND[k] /= 1);
        k = k + 1;
    endo;
    psik0 = zeros(N_OBS, N_INSIDE);
    psi1 = zeros(N_OBS, N_INSIDE);
    psi2 = zeros(N_OBS, N_INSIDE);
    psi3 = zeros(N_OBS, N_INSIDE);
    k = 1 ;
    do while k <= N_PSI;
        psik0 = psik0 + ( ppsi[k].*ones(N_OBS,1) )
               .* PSIDATA[.,((k-1)*N_INSIDE+1):(k*N_INSIDE)];
        psi1 = psi1 + ( ppsi[k].*ones(N_OBS,1) )
               .* PSIDATA1[.,((k-1)*N_INSIDE+1):(k*N_INSIDE)];
        psi2 = psi2 + ( ppsi[k].*ones(N_OBS,1) )
               .* PSIDATA2[.,((k-1)*N_INSIDE+1):(k*N_INSIDE)];
        psi3 = psi3 + ( ppsi[k].*ones(N_OBS,1) )
               .* PSIDATA3[.,((k-1)*N_INSIDE+1):(k*N_INSIDE)];
        k = k + 1;
    endo;  
    wtpall    = zeros(N_OBS,3);   
    demandall = zeros(N_OBS,4);
    demandall2 = zeros(N_OBS,4*N_INSIDE);
    lambdaball = zeros(N_OBS,5);

    x2i0 = Max_TS - sumc((TTIME.*SK.*XINSIDE)');
    x3i0 = Max_TL - sumc((TTIME.*(1-SK).*XINSIDE)');
    eps1 = erre0[.,1];
    eps2 = erre0[.,2];
    eps3 = erre0[.,3];    
    gumbelk0 = ( -psik0 + ln(PRICE.*XOUT1^(alpha1-1).*exp(xi1+eps1) 
	        + XI_DUMMY[2].* exp(xis+eps2).*TTIME.*sk.*x2i^(alpha2-1)
			+ XI_DUMMY[3].* exp(xil+eps3).*TTIME.*(1-sk).*x3i^(alpha3-1))
            + (1-alphak).*my_ln(XINSIDE ./ gammak + 1) )./sigk;  
    likek0 = (XINSIDE./=0).*(-gumbelk0-my_ln(sigk))-my_exp(-gumbelk0); 
    likek0 = my_exp( sumc((CSET_INSIDE.*likek0)' ) );
    det0 = zeros(N_OBS,1);
    s = 1;
    do while s <= N_SIM;
        erre1 = erre0 + rho.*( (sig123'.*(sig_RAND .== 1)').*rndns(N_OBS,rows(sig123),NSEED2) );      
        pdf1 = ones(N_OBS,1);     
        k = 1;
        do while k <= rows(sig123);
            pdf1 = pdf1.*(sig_RAND[k] .== 1)
                   .*1./sig123[k].*exp(-erre1[.,k]./sig123[k]).*(exp(-exp(-erre1[.,k]./sig123[k]) )) 
                   + pdf1.*(sig_RAND[k] ./= 1);
            k = k + 1;
        endo;
        erre1 = erre1.*(sig_RAND' .==1);
        eps1 = erre1[.,1];
        eps2 = erre1[.,2];
        eps3 = erre1[.,3];
        det1 = zeros(N_OBS,1);
        k = 1;
        do while k <= N_OBS;
          if sumc((XINSIDE[k,.].*CSET_INSIDE[k,.])') .eq 0;
            det1[k] = 1;
          else;  			
            det2 = ( ( ((1-alpha1).*(PRICE[k,.]')*PRICE[k,.].*XOUT1[k]^(alpha1-2).*exp(xi1+eps1[k])
                 + (1-alpha2).*((TTIME[k,.]')*TTIME[k,.]).* XI_DUMMY[2].* exp(xis+eps2[k]).* x2i[k]^(alpha2-2).*((sk[k,.]')*sk[k,.])
                 + (1-alpha3).*((TTIME[k,.]')*TTIME[k,.]).* XI_DUMMY[3].* exp(xil+eps3[k]).* x3i[k]^(alpha3-2).*((sk2[k,.]')*sk2[k,.])
                 ) ./          
                 (PRICE[k,.].*XOUT1[k]^(alpha1-1).*exp(xi1+eps1[k])
                 + XI_DUMMY[2].* exp(xis+eps2[k]).*TTIME[k,.].* sk[k,.].*x2i[k]^(alpha2-1) 
                 + XI_DUMMY[3].* exp(xil+eps3[k]).*TTIME[k,.].* sk2[k,.].*x3i[k]^(alpha3-1))
               .*ones(N_INSIDE,N_INSIDE)+ 
               (eye(N_INSIDE).*((1-alphak)./(XINSIDE[k,.] + 
               gammak[k,.]))) ).*(XINSIDE[k,.] ./= 0) ).*
               ( (XINSIDE[k,.].*CSET_INSIDE[k,.] ./=0) )';           
            det2 = (XINSIDE[k,.].*CSET_INSIDE[k,.] .== 0).*eye(N_INSIDE) + det2;          
            det1[k] = abs(det(det2));
	      endif;
          k = k + 1;	
        endo;       
        gumbelk1 = ( -psik0 + ln(PRICE.*XOUT1^(alpha1-1).*exp(xi1+eps1) 
	        + XI_DUMMY[2].* exp(xis+eps2).*TTIME.*SK.*x2i0^(alpha2-1)
			+ XI_DUMMY[3].* exp(xil+eps3).*TTIME.*(1-SK).*x3i0^(alpha3-1))
            + (1-alphak).*my_ln(XINSIDE ./ gammak + 1) )./sigk;  
        like1 = (XINSIDE./=0).*(-gumbelk1-my_ln(sigk))-my_exp(-gumbelk1); 
        likek1 = det1.*my_exp( sumc((CSET_INSIDE.*like1)' ) );
        stat = my_exp(my_ln(pdf1)+my_ln(likek1)-my_ln(pdf0)-
           my_ln(likek0));    
        thresh = rndus(N_OBS,1,NSEED2);
        erre0 = (stat.<thresh).*erre0 + (stat.>=thresh).*erre1;
        likek0 = (stat.<thresh).*likek0 + (stat.>=thresh).*likek1;
        det0 = (stat.<thresh).*det0 + (stat.>=thresh).*det1;
        pdf0 = (stat.<thresh).*pdf0 + (stat.>=thresh).*pdf1;        
        gumbelk0 = (stat.<thresh).*gumbelk0 +(stat.>=thresh).*gumbelk1;
        rho = rho*(1.025)*( meanc((stat.>=thresh)) >= .3 ) 
                + rho*(0.975)*( meanc((stat.>=thresh)) <  .3 );
        eps1 = erre0[.,1];
        eps2 = erre0[.,2];
        eps3 = erre0[.,3];
        
        if (s.*(((s-N_BURNIN)./SKIP1) == trunc((s-N_BURNIN)./SKIP1)) <= N_BURNIN) .* ((s % 100) == 0);
            if N_KR == 1;
                print s "BURN-IN";
            endif;
        endif;
        if s.*(((s-N_BURNIN)./SKIP1) == trunc((s-N_BURNIN)./SKIP1)) > N_BURNIN;
            if N_KR == 1;
                print s ;             
            endif;
            eps = eps1~eps2~eps3;
            errk = (XINSIDE .> 0).*( gumbelk0.*(sigk) );
            errk = errk + (XINSIDE .== 0).*( (-my_ln(-my_ln( 				
                rndus(N_OBS,N_INSIDE,NSEED1).*(my_exp(-my_exp(-gumbelk0))) ))).*	
                (sigk) );			
            psikn = my_exp(psik0 + errk);        
            psi1n = my_exp(psi1 + errk);
            psi2n = my_exp(psi2 + errk); 
            psi3n = my_exp(psi3 + errk);    
           
        @ --- Compensating Variations (WTP/WTA) --- @
      
        @ Baseline Condition @
            struct util_data ud0;
                ud0.demand = XINSIDE;
                ud0.price = PRICE;
                ud0.inc = INCOME;
                ud0.alpha1 = alpha1;
                ud0.alpha2 = alpha2;
                ud0.alpha3 = alpha3;
                ud0.alphak = alphak;
                ud0.gammak = gammak;
                ud0.psik = psikn;
                ud0.xi1 = xi1;
                ud0.xis = xis;
                ud0.xil = xil;
                ud0.maxts = max_ts;
                ud0.maxtl = max_tl;
                ud0.tt = TTIME;
                ud0.keep = CSET_INSIDE;
                ud0.eps = eps;
                utilb = dutil(ud0);

        @ SCENARIO #1 @
            struct demand_data md;
            md.psi = psi1n;
            md.price = PRICE1;
            md.util0 = utilb;
            md.alpha1 = alpha1;
            md.alpha2 = alpha2;
            md.alpha3 = alpha3;
            md.alphak = alphak;
            md.gamma = gammak;
            md.keep = CSET_INSIDE1;
            md.hprice = HICKSP;
            md.eps = eps;
            md.maxts = Max_TS1;
            md.maxtl = Max_TL1;
            md.xi1 = xi1;
            md.xis = xis;
            md.xil = xil;
            md.tt = TTIME1;
            {demand1i,hlambda1i} = cdemand(md);
            wtp1i = INCOME-sumc(((PRICE1.*CSET_INSIDE~HICKSP).*demand1i)');

        @ SCENARIO #2 @
            md.psi = psi2n;
            md.price = PRICE2;
            md.util0 = utilb;
            md.alpha1 = alpha1;
            md.alpha2 = alpha2;
            md.alpha3 = alpha3;
            md.alphak = alphak;
            md.gamma = gammak;
            md.keep = CSET_INSIDE2;
            md.hprice = HICKSP;
            md.eps = eps;
            md.maxts = Max_TS2;
            md.maxtl = Max_TL2;
            md.xi1 = xi1;
            md.xis = xis;
            md.xil = xil;
            md.tt = TTIME2;
            {demand2i,hlambda2i} = cdemand(md);
            wtp2i = INCOME-sumc(((PRICE2.*CSET_INSIDE2~HICKSP).*demand2i)');	
   
        @ SCENARIO #3 @
            md.psi = psi3n;
            md.price = PRICE3;
            md.util0 = utilb;
            md.alpha1 = alpha1;
            md.alpha2 = alpha2;
            md.alpha3 = alpha3;
            md.alphak = alphak;
            md.gamma = gammak;
            md.keep = CSET_INSIDE3;
            md.hprice = HICKSP;
            md.eps = eps;
            md.maxts = Max_TS3;
            md.maxtl = Max_TL3;
            md.xi1 = xi1;
            md.xis = xis;
            md.xil = xil;
            md.tt = TTIME3;
            {demand3i,hlambda3i} = cdemand(md);
            wtp3i = INCOME-sumc(((PRICE3.*CSET_INSIDE3~HICKSP).*demand3i)');	
   
            wtpall = wtpall + (wtp1i~wtp2i~wtp3i)./((N_SIM-N_BURNIN)./SKIP1);

        @ --- Demand Calculations --- @

        @ Baseline Condition @
            md.psi = psikn;
            md.price = PRICE;
            md.inc = INCOME;
            md.alpha1 = alpha1;
            md.alpha2 = alpha2;
            md.alpha3 = alpha3;
            md.alphak = alphak;
            md.gamma = gammak;
            md.keep = CSET_INSIDE;
            md.hprice = HICKSP;
            md.eps = eps;
            md.maxts = Max_TS;
            md.maxtl = Max_TL;
            md.xi1 = xi1;
            md.xis = xis;
            md.xil = xil;
            md.tt = TTIME;
            md.lambda = 0;
            {demandbi, lambdabi, retcode} = udemand(md);

        @ SCENARIO #1 @
            md.psi = psi1n;
            md.price = PRICE1;
            md.inc = INCOME;
            md.alpha1 = alpha1;
            md.alpha2 = alpha2;
            md.alpha3 = alpha3;
            md.alphak = alphak;
            md.gamma = gammak;
            md.keep = CSET_INSIDE1;
            md.hprice = HICKSP;
            md.eps = eps;
            md.maxts = Max_TS1;
            md.maxtl = Max_TL1;
            md.xi1 = xi1;
            md.xis = xis;
            md.xil = xil;
            md.tt = TTIME1;
            md.lambda = lambdabi;
            {demand1i, lambda1i, retcode} = udemand(md);
 
        @ SCENARIO #2 @
            md.psi = psi2n;
            md.price = PRICE2;
            md.inc = INCOME;
            md.alpha1 = alpha1;
            md.alpha2 = alpha2;
            md.alpha3 = alpha3;
            md.alphak = alphak;
            md.gamma = gammak;
            md.keep = CSET_INSIDE2;
            md.hprice = HICKSP;
            md.eps = eps;
            md.maxts = Max_TS2;
            md.maxtl = Max_TL2;
            md.xi1 = xi1;
            md.xis = xis;
            md.xil = xil;
            md.tt = TTIME2;
            md.lambda = lambdabi;
            {demand2i, lambda2i, retcode} = udemand(md);

        @ SCENARIO #3 @
            md.psi = psi3n;
            md.price = PRICE3;
            md.inc = INCOME;
            md.alpha1 = alpha1;
            md.alpha2 = alpha2;
            md.alpha3 = alpha3;
            md.alphak = alphak;
            md.gamma = gammak;
            md.keep = CSET_INSIDE3;
            md.hprice = HICKSP;
            md.eps = eps;
            md.maxts = Max_TS3;
            md.maxtl = Max_TL3;
            md.xi1 = xi1;
            md.xis = xis;
            md.xil = xil;
            md.tt = TTIME3;
            md.lambda = lambdabi;
            {demand3i, lambda3i, retcode} = udemand(md);

            demandall = demandall + (sumc(demandbi')~sumc(demand1i')~sumc(
                demand2i')~sumc(demand3i'))./((N_SIM-N_BURNIN)./SKIP1);
            demandall2 = demandall2 + (demandbi~demand1i~demand2i~demand3i)
                ./((N_SIM-N_BURNIN)./SKIP1);
            lambdaball = lambdaball + (lambdabi~lambdabi[.,2]./lambdabi[.,1]~lambdabi[.,3]./lambdabi[.,1])
                ./((N_SIM-N_BURNIN)./SKIP1);
        endif;
        s = s + 1;
    endo;
    retp(wtpall,demandall,demandall2,lambdaball);   
endp;
         

@ Procedures for Solving Uncompensated Demand @

proc (3) = udemand(struct demand_data md);
    local demand, lambda1, lambda2, lambda3;
    local n, ret, start, tcode, tcodeall;
    demand = zeros(N_OBS,N_INSIDE);
    lambda1 = zeros(N_OBS,1);
    lambda2 = zeros(N_OBS,1);
    lambda3 = zeros(N_OBS,1);
    tcodeall = zeros(N_OBS,1);
    n = 1;
    do while n .le N_OBS;
            dd_psin = md.psi[n,.];
            dd_pricen = md.price[n,.];
            dd_alpha1n = md.alpha1;
            dd_alpha2n = md.alpha2;
            dd_alpha3n = md.alpha3;
            dd_alphakn = md.alphak;
            dd_gamman = md.gamma[n,.];
            dd_incn = md.inc[n,.];
            dd_keepn = md.keep[n,.];
            dd_maxtsn = md.maxts[n,.];
            dd_maxtln = md.maxtl[n,.];
            dd_xi1n = md.xi1;
            dd_xisn = md.xis;
            dd_xiln = md.xil;
            dd_ttn = md.tt[n,.]; 
            dd_skn = SK[n,.];
            dd_eps1n = md.eps[n,1];
            dd_eps2n = md.eps[n,2];
            dd_eps3n = md.eps[n,3];
        start = zeros(N_CONSTRAINTS,1);
        start[1] = exp(dd_xi1n+dd_eps1n).* (XOUT1[n])^(dd_alpha1n-1) ;
        if N_CONSTRAINTS .>= 2;
            start[2] = exp(dd_xisn+dd_eps2n).*(dd_maxtsn - sumc((dd_ttn.*dd_skn.*XINSIDE[n,.])'))^(dd_alpha2n-1);
        endif;
        if N_CONSTRAINTS .>= 3;
            start[3] = exp(dd_xiln+dd_eps3n).*(dd_maxtln - sumc((dd_ttn.*(1-dd_skn).*XINSIDE[n,.])'))^(dd_alpha3n-1);
        endif;
        eqsolveset;
        __output = 0;
        {ret, tcode} = eqSolve(&f_udemand, start);
        tcodeall[n,1] = tcode;
        lambda1[n,1] = ret[1];
        if N_CONSTRAINTS .>= 2;
            lambda2[n,1] = ret[2];
        endif;
        if N_CONSTRAINTS .>= 3;
            lambda3[n,1] = ret[3];
        endif;
        demand[n,.] =dd_gamman.* (dd_psin ./ (dd_pricen .* lambda1[n,1] + lambda2[n,1].*dd_ttn.*dd_skn
                    +  lambda3[n,1].*dd_ttn.*(1-dd_skn))).^(1/(1-dd_alphakn)) - dd_gamman;
        demand[n,.] = dd_keepn.*(demand[n,.] .>= 0).*demand[n,.];
        n = n + 1;      
    endo;
    retp(demand, lambda1~lambda2~lambda3, tcodeall);      
endp;

proc f_udemand(x);
    local eqns, demand;
    x[1] = x[1] ;
    if N_CONSTRAINTS .== 3;
        demand = dd_gamman.* ( dd_psin ./ ( dd_pricen .* x[1] + x[2].* dd_ttn.* dd_skn
                + x[3].* dd_ttn.*(1- dd_skn))).^(1/(1- dd_alphakn)) -  dd_gamman;
        demand =  dd_keepn.*(demand .>= 0).*demand;
        eqns = zeros(3,1);
        eqns[1] =  dd_incn - sumc(( dd_pricen.*demand)') - (exp( dd_xi1n+ dd_eps1n) ./x[1])^(1./(1- dd_alpha1n));
        eqns[2] =  dd_maxtsn - sumc(( dd_ttn.* dd_skn.*demand)')  - (exp( dd_eps2n+ dd_xisn)./x[2])^(1./(1- dd_alpha2n));
        eqns[3] =  dd_maxtln - sumc(( dd_ttn.*(1- dd_skn).*demand)') - (exp( dd_eps3n+ dd_xiln)./x[3])^(1./(1- dd_alpha3n));
    elseif N_CONSTRAINTS .== 2;
        demand = dd_gamman.* ( dd_psin ./ ( dd_pricen .* x[1] + x[2].* dd_ttn
                ))^(1/(1- dd_alphakn)) -  dd_gamman;
        demand =  dd_keepn.*(demand .>= 0).*demand;
        eqns = zeros(2,1);
        eqns[1] =  dd_incn - sumc(( dd_pricen.*demand)') - (exp( dd_xi1n+ dd_eps1n) ./x[1])^(1./(1- dd_alpha1n));
        eqns[2] =  dd_maxtsn - sumc(( dd_ttn.* demand)')  - (exp( dd_eps2n+ dd_xisn)./x[2])^(1./(1- dd_alpha2n));
    elseif N_CONSTRAINTS .== 1;
        demand = dd_gamman.* ( dd_psin ./ ( dd_pricen .* x[1] ))^(1/(1- dd_alphakn)) -  dd_gamman;
        demand =  dd_keepn.*(demand .>= 0).*demand;
        eqns = zeros(1,1);
        eqns[1] =  dd_incn - sumc(( dd_pricen.*demand)') - (exp( dd_xi1n+ dd_eps1n) ./x[1])^(1./(1- dd_alpha1n));
    endif;
        retp(eqns);
endp;

@ Procedures for Solving Compensated Demands @
proc (2) = cdemand(struct demand_data md);
    local demand, hlambda1, hlambda2, hlambda3, x1h;
    local n, ret, start, tcode;
    demand = zeros(N_OBS,N_INSIDE);
    hlambda1 = zeros(N_OBS,1);
    hlambda2 = zeros(N_OBS,1);
    hlambda3 = zeros(N_OBS,1);
    x1h = zeros(N_OBS,1);
    n = 1;
    do while n .le N_OBS;
        dd_psin = md.psi[n,.];
        dd_pricen = md.price[n,.];
        dd_alpha1n = md.alpha1;
        dd_alpha2n = md.alpha2;
        dd_alpha3n = md.alpha3;
        dd_alphakn = md.alphak;
        dd_gamman = md.gamma[n,.];
        dd_keepn = md.keep[n,.];
        dd_maxtsn = md.maxts[n,.];
        dd_maxtln = md.maxtl[n,.];
        dd_xi1n = md.xi1;
        dd_xisn = md.xis;
        dd_xiln = md.xil;
        dd_ttn = md.tt[n,.];
        dd_skn = SK[n,.]; 
        dd_eps1n = md.eps[n,1];
        dd_eps2n = md.eps[n,2];
        dd_eps3n = md.eps[n,3];
        dd_util0n = md.util0[n,.];
        start = zeros(N_CONSTRAINTS,1);
        start[1] = exp(dd_xi1n+dd_eps1n).* (XOUT1[n])^(dd_alpha1n-1);
        if N_CONSTRAINTS .>= 2;
            start[2] = exp(dd_xisn+dd_eps2n).*(dd_maxtsn - sumc((dd_ttn.*dd_skn.*XINSIDE[n,.])'))^(dd_alpha2n-1);
        endif;
        if N_CONSTRAINTS .>= 3;
            start[3] = exp(dd_xiln+dd_eps3n).*(dd_maxtln - sumc((dd_ttn.*(1-dd_skn).*XINSIDE[n,.])'))^(dd_alpha3n-1);
        endif;
        eqsolveset;          
        __output = 0;

        {ret, tcode} = eqSolve(&f_cdemand, start);      
        hlambda1[n,1] = ret[1];
        if N_CONSTRAINTS .>= 2;
            hlambda2[n,1] = ret[2];
        endif;
        if N_CONSTRAINTS .>= 3;
            hlambda3[n,1] = ret[3];
        endif;
        demand[n,.] =dd_gamman.* (dd_psin ./ (dd_pricen .* hlambda1[n,1] + hlambda2[n,1].*dd_ttn.*dd_skn
                    +  hlambda3[n,1].*dd_ttn.*(1-dd_skn))).^(1/(1-dd_alphakn)) - dd_gamman;
        demand[n,.] = dd_keepn.*(demand[n,.] .>= 0).*demand[n,.];   
        x1h[n,.] = (exp(dd_xi1n+dd_eps1n) ./hlambda1[n,1]).^(1./(1-dd_alpha1n));
        n = n + 1;
   endo;
   retp(demand~x1h,hlambda1~hlambda2~hlambda3);   
endp;

proc f_cdemand(xx);
    local eqns, demand, utiln, x1hn, ux1n, ux2n, ux3n, uxkn, x;
    xx = (xx.>1e-10).*xx + (xx.<=1e-10).*1e-10;
    x = zeros(3,1);
    if N_CONSTRAINTS .== 3;
        x = xx;
    elseif N_CONSTRAINTS .== 2;
        x[1] = xx[1];
        x[2] = xx[2];
    elseif N_CONSTRAINTS .== 1;
        x[1] = xx[1];
    endif;
    demand = dd_gamman.* (dd_psin ./ (dd_pricen .* x[1] + x[2].*dd_ttn.*dd_skn
            + x[3].*dd_ttn.*(1-dd_skn))).^(1/(1-dd_alphakn)) - dd_gamman;
	demand = dd_keepn.*(demand .>= 0).*demand;

    x1hn = ( exp(dd_xi1n+dd_eps1n) ./x[1])^(1./(1-dd_alpha1n)); 

    if dd_alpha1n ./= 0;
        ux1n =  (exp(dd_xi1n+dd_eps1n)./dd_alpha1n).*((exp(dd_xi1n+dd_eps1n) ./x[1]).^(dd_alpha1n./(1-dd_alpha1n)));
    else;
        ux1n =  exp(dd_xi1n+dd_eps1n).*ln(exp(dd_xi1n+dd_eps1n)./x[1]);        
    endif;
    if N_CONSTRAINTS .>= 2;
        if dd_alpha2n ./= 0;
            ux2n =  (exp(dd_xisn+dd_eps2n)./dd_alpha2n).*((exp(dd_xisn+dd_eps2n) ./x[2]).^(dd_alpha2n./(1-dd_alpha2n)));
        else;
            ux2n =  exp(dd_xisn+dd_eps2n).*ln(exp(dd_xisn+dd_eps2n)./x[2]);        
        endif;
    else;
        ux2n = 0;
    endif;
    if N_CONSTRAINTS .>= 3;
        if dd_alpha3n ./= 0;
            ux3n =  (exp(dd_xiln+dd_eps3n)./dd_alpha3n).*((exp(dd_xiln+dd_eps3n) ./x[3]).^(dd_alpha3n./(1-dd_alpha3n)));
        else;
            ux3n =  exp(dd_xiln+dd_eps3n).*ln(exp(dd_xiln+dd_eps3n)./x[3]);        
        endif;
    else;
        ux3n = 0;
    endif;
    if dd_alphakn[1] ./= 0;
       uxkn =  sumc((dd_gamman.*dd_psin.*dd_keepn./dd_alphakn
          .*((demand./dd_gamman + 1).^dd_alphakn - 1))');
    else;
       uxkn =  sumc((dd_gamman.*dd_psin.*dd_keepn.*ln(demand./dd_gamman + 1))');        
    endif;    
    utiln = ux1n.*XI_DUMMY[1] + ux2n.*XI_DUMMY[2] + ux3n.*XI_DUMMY[3] + uxkn;
    eqns = zeros(N_CONSTRAINTS,1);
    eqns[1] = utiln - dd_util0n;
    if N_CONSTRAINTS .>=2;
        eqns[2] = dd_maxtsn - sumc((dd_ttn.*dd_skn.*demand)')  - (exp(dd_eps2n+dd_xisn)./x[2]).^(1./(1-dd_alpha2n));
    endif;
    if N_CONSTRAINTS .>=3;
        eqns[3] = dd_maxtln - sumc((dd_ttn.*(1-dd_skn).*demand)') - (exp(dd_eps3n+dd_xiln)./x[3]).^(1./(1-dd_alpha3n));
    endif;
    retp(eqns);
endp;

@ Procedure for Calculating Direct Utility Function @

proc dutil(struct util_data ud0);
    local utility, ux1, ux2,ux3, uxk;  
    if ALPHA_EST[1] ./= 0;
        ux1 = (exp(ud0.xi1+ud0.eps[.,1])./ud0.alpha1).*(ud0.inc-sumc((ud0.price.*ud0.demand.*ud0.keep)'))^ud0.alpha1;
    else;
        ux1 = exp(ud0.xi1+ud0.eps[.,1]).*ln(ud0.inc-sumc((ud0.price.*ud0.demand.*ud0.keep)'));
    endif;
    if ALPHA_EST[2] ./= 0;
        ux2 = (exp(ud0.xis+ud0.eps[.,2])./ud0.alpha2).*(ud0.maxts-sumc((ud0.tt.*SK.*ud0.demand.*ud0.keep)'))^ud0.alpha2;
    else;
        ux2 = exp(ud0.xis+ud0.eps[.,2]).*ln(ud0.maxts-sumc((ud0.tt.*SK.*ud0.demand.*ud0.keep)'));
    endif;
    if ALPHA_EST[3] ./= 0;
        ux3 = (exp(ud0.xil+ud0.eps[.,3])./ud0.alpha3).*(ud0.maxtl-sumc((ud0.tt.*(1-SK).*ud0.demand.*ud0.keep)'))^ud0.alpha3;
    else;
        ux3 = exp(ud0.xil+ud0.eps[.,3]).*ln(ud0.maxtl-sumc((ud0.tt.*(1-SK).*ud0.demand.*ud0.keep)'));
    endif;    
    if ALPHA_EST[4] ./= 0;
        uxk = sumc( (ud0.gammak.*ud0.psik.*ud0.keep./ud0.alphak
            .*((ud0.demand./ud0.gammak + ones(N_OBS,1)).^ud0.alphak - ones(N_OBS,1)))');
    else;
        uxk = sumc(( ud0.gammak.*ud0.psik.*ud0.keep.*ln(ud0.demand./ud0.gammak + ones(N_OBS,1)))');
    endif;   
    utility = ux1.*XI_DUMMY[1] + ux2.*XI_DUMMY[2] + ux3.*XI_DUMMY[3] + uxk;
    retp(utility);
endp;

@ Procedure for 95% confidence intervals @
proc (2) = confint(x);
    local xl, xu;
    x = sortc(x,1);
    xl = ceil(rows(x)*.025);
    xu = ceil(rows(x)*(1-.025));
    retp(x[xl,1],x[xu,1]);
endp;

@ My Procedure for Generating ln(x) @

proc my_ln(x);
    local y;
    y = (x.<1e-250).*1e-250 + (x.>=1e-250).*x;
    retp(ln(y));
endp;


@ My Procedure for Generating exp(x) @

proc my_exp(x);
    local y;
    y = (x.<-700).*(-700)+(abs(x).<=700).*x+(x.>=700).*(700);
    retp(exp(y));
endp;


/*
******************************************************************
              Point Estimates of Demand, WTP, and VLT            
******************************************************************

Time (in minutes):                       0.34480000 
Total Generated Draws:                     2500.0000 
Discarded as Burn-in:                     500.00000 
Using Every nth Draw,          n =:       10.000000 
Total Draws Used for Solving:             200.00000 


Mean for Demand
       BASELINE         SCENARIO #1      SCENARIO #2      SCENARIO #3
       19.680000        19.804277        19.889563        18.452327 


       GOODS            BASELINE         SCENARIO #1      SCENARIO #2      SCENARIO #3

       1.0000000        2.0800000        2.1316642        2.0736393        2.1435594 
       2.0000000        2.2800000        2.3291306        2.2631460        2.3976008 
       3.0000000        3.0600000        3.1493116        3.0309821        3.1944100 
       4.0000000        3.1800000        3.1698127        3.2358056       0.00000000 
       5.0000000        4.3500000        4.3332722        4.3965689        4.6798252 
       6.0000000        4.7300000        4.6910855        4.8894207        6.0369317 


Mean for WTP
       SCENARIO #1      SCENARIO #2      SCENARIO #3
       965.15296        80.670669       -3866.3656 


Mean for Lambda1      Lambda2         Lambda3             VLTS             VLTL
       11017.095     0.0027462800    0.00016954569        8.8537838       0.75378398 

******************************************************************
              Krinsky-Robb Monte Carlo Simulation                 
******************************************************************
Time (in minutes):                        71.803167 
Number of Krinsky-Robb Iterations:        200.00000 
Total Genrated Draws:                     2500.0000 
Discarded as Burn-in:                     500.00000 
Using Every nth Draw,          n =:       10.000000 
Total Draws Used for Solving:             200.00000 


Demand (Scenario #0 means the baseline)
       SCENARIO           MEAN            SD               95%lower         95%upper

      0.00000000        19.680000   7.7112762e-014        19.680000        19.680000 
       1.0000000        19.804259     0.0062450599        19.792238        19.816010 
       2.0000000        19.892238      0.037475630        19.819837        19.968505 
       3.0000000        18.455862      0.035137415        18.388644        18.520357 


Demand for Each Good (Scenario #0 means the baseline)
       SCENARIO           GOOD              MEAN            SD             95%lower         95%upper

      0.00000000        1.0000000        2.0800000   2.7600829e-015        2.0800000        2.0800000 
      0.00000000        2.0000000        2.2800000   1.1492076e-014        2.2800000        2.2800000 
      0.00000000        3.0000000        3.0600000   5.9647316e-016        3.0600000        3.0600000 
      0.00000000        4.0000000        3.1800000   2.1955355e-015        3.1800000        3.1800000 
      0.00000000        5.0000000        4.3500000   6.4602097e-015        4.3500000        4.3500000 
      0.00000000        6.0000000        4.7300000   1.4888983e-014        4.7300000        4.7300000 
       1.0000000        1.0000000        2.1317238     0.0015482111        2.1285729        2.1346424 
       1.0000000        2.0000000        2.3285013     0.0023227121        2.3220944        2.3318861 
       1.0000000        3.0000000        3.1492775     0.0019323695        3.1454153        3.1527411 
       1.0000000        4.0000000        3.1699341    0.00092861425        3.1681900        3.1717024 
       1.0000000        5.0000000        4.3332579    0.00070619618        4.3317684        4.3346216 
       1.0000000        6.0000000        4.6915641     0.0033148575        4.6856339        4.6980154 
       2.0000000        1.0000000        2.0735135     0.0011102865        2.0709626        2.0755623 
       2.0000000        2.0000000        2.2627118     0.0032177830        2.2550440        2.2683124 
       2.0000000        3.0000000        3.0303978     0.0050503870        3.0186531        3.0396778 
       2.0000000        4.0000000        3.2370794     0.0098761719        3.2185599        3.2563313 
       2.0000000        5.0000000        4.3953901     0.0098496946        4.3768868        4.4125836 
       2.0000000        6.0000000        4.8931455      0.028716240        4.8385924        4.9499605 
       3.0000000        1.0000000        2.1433635     0.0020823884        2.1393051        2.1478730 
       3.0000000        2.0000000        2.3974218     0.0038909879        2.3900764        2.4050912 
       3.0000000        3.0000000        3.1935220     0.0078913828        3.1786082        3.2091781 
       3.0000000        4.0000000       0.00000000       0.00000000       0.00000000       0.00000000 
       3.0000000        5.0000000        4.6806930     0.0054073835        4.6714075        4.6926350 
       3.0000000        6.0000000        6.0408621      0.041004957        5.9604145        6.1191802 


WTP
       SCENARIO           MEAN            SD               95%lower         95%upper

       1.0000000        979.85590        85.065189        819.69249        1155.5104 
       2.0000000        85.915965        23.954699        48.242819        139.09158 
       3.0000000       -2181.1540        1414.7084       -4412.4106       -115.89116 


Lambda and Time Value
#1:lambda1, #2:lambda2, #3:lambda3, #4:VLTS, #5:VLSL
           #             MEAN             SD             95%lower           95%upper

       1.0000000        11920.954        11259.849        2666.1398        42123.393 
       2.0000000     0.0028979840    0.00091952116     0.0014035188     0.0048564284 
       3.0000000    0.00018033062   5.2552900e-005    0.00010087061    0.00028915089 
       4.0000000        9.0467993       0.87220575        7.4417193        10.863610 
       5.0000000       0.80647664       0.23388948       0.44734022        1.3225860 

*/
